<?php

namespace Drupal\Tests\components\Kernel;

/**
 * @coversDefaultClass \Drupal\components\Template\TwigExtension
 * @group components
 */
class TwigExtensionTest extends ComponentsKernelTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'components',
    'components_twig_extension_test',
  ];

  /**
   * Ensures the Twig template() function works inside a Drupal instance.
   *
   * @covers ::template
   *
   * @throws \Exception
   */
  public function testTemplateFunction() {
    $result = NULL;
    try {
      $element = [
        '#theme' => 'components_twig_extension_test_template_function',
        '#items' => [
          'first item',
          'second item',
        ],
      ];
      $result = $this->render($element);
    }
    catch (\Exception $e) {
      $this->fail('No Exception expected; "' . $e->getMessage() . '" thrown during: ' . $this->getName());
    }
    $expected = '<ul><li>first item</li><li>second item</li></ul>';
    if (method_exists($this, 'assertStringContainsString')) {
      $this->assertStringContainsString($expected, $result);
    }
    else {
      $this->assertContains($expected, $result);
    }
  }

  /**
   * Ensures the Twig "recursive_merge" filter works inside a Drupal instance.
   *
   * @covers ::recursiveMergeFilter
   */
  public function testRecursiveMergeFilter() {
    try {
      $element = [
        '#theme' => 'components_twig_extension_test_recursive_merge_filter',
        'list' => [
          '#theme' => 'item_list',
          '#items' => [
            [
              '#type' => 'container',
              '#attributes' => [
                'id' => 'the_element_id',
                'class' => ['original-container-class'],
              ],
            ],
          ],
        ],
      ];
      $result = $this->render($element);
    }
    catch (\Exception $e) {
      $this->fail('No Exception expected; "' . $e->getMessage() . '" thrown during: ' . $this->getName());
    }
    $this->assertStringContainsString('<div id="the_element_id" class="new-class"></div>', $result);
  }

  /**
   * Ensures the Twig "set" filter works inside a Drupal instance.
   *
   * @covers ::setFilter
   */
  public function testSetFilter() {
    $result = NULL;
    try {
      $element = [
        '#theme' => 'components_twig_extension_test_set_filter',
        'list' => [
          '#theme' => 'item_list',
          '#items' => [
            [
              '#type' => 'container',
              '#attributes' => [
                'id' => 'the_element_id',
                'class' => ['original-container-class'],
              ],
            ],
          ],
        ],
      ];
      $result = $this->render($element);
    }
    catch (\Exception $e) {
      $this->fail('No Exception expected; "' . $e->getMessage() . '" thrown during: ' . $this->getName());
    }
    $expected = '<div class="new-class"></div>';
    if (method_exists($this, 'assertStringContainsString')) {
      $this->assertStringContainsString($expected, $result);
    }
    else {
      $this->assertContains($expected, $result);
    }
  }

  /**
   * Ensures the Twig "add" filter works inside a Drupal instance.
   *
   * @covers ::addFilter
   */
  public function testAddFilter() {
    $result = NULL;
    try {
      $element = [
        '#theme' => 'components_twig_extension_test_add_filter',
        'list' => [
          '#theme' => 'item_list',
          '#items' => [
            [
              '#type' => 'container',
              '#attributes' => [
                'class' => ['original-container-class'],
              ],
            ],
          ],
        ],
      ];
      $result = $this->render($element);
    }
    catch (\Exception $e) {
      $this->fail('No Exception expected; "' . $e->getMessage() . '" thrown during: ' . $this->getName());
    }
    $expected = '<div class="original-container-class new-class"></div>';
    if (method_exists($this, 'assertStringContainsString')) {
      $this->assertStringContainsString($expected, $result);
    }
    else {
      $this->assertContains($expected, $result);
    }
  }

}
